@@ -129,7 +129,8 @@ with some simple extensions.")
 
    "g" gnus-server-regenerate-server
 
-    "\C-c\C-i" gnus-info-find-node))
+    "\C-c\C-i" gnus-info-find-node
+    "\C-c\C-b" gnus-bug))
 
 (defun gnus-server-mode ()
   "Major mode for listing and editing servers.
@@ -206,15 +207,16 @@ The following commands are available:
     (setq gnus-inserted-opened-servers nil)
     ;; First we do the real list of servers.
     (while alist
-      (push (caar alist) done)
-      (cdr (setq server (pop alist))) 
-      (when (and server (car server) (cdr server))
-	(gnus-server-insert-server-line (car server) (cdr server))))
+      (unless (member (cdar alist) done)
+	(push (cdar alist) done)
+	(cdr (setq server (pop alist))) 
+	(when (and server (car server) (cdr server))
+	  (gnus-server-insert-server-line (car server) (cdr server)))))
     ;; Then we insert the list of servers that have been opened in
     ;; this session.
     (while opened 
-      (unless (member (cadaar opened) done)
-	(push (cadaar opened) done)
+      (unless (member (caar opened) done)
+	(push (caar opened) done)
 	(gnus-server-insert-server-line 
 	 (setq op-ser (format "%s:%s" (caaar opened) (nth 1 (caar opened))))
 	 (caar opened))
@@ -241,8 +243,7 @@ The following commands are available:
       (when entry
 	(gnus-dribble-enter 
 	 (concat "(gnus-server-set-info \"" server "\" '"
-		 (prin1-to-string (cdr entry)) ")
-")))
+		 (prin1-to-string (cdr entry)) ")\n")))
       (when (or entry oentry)
 	;; Buffer may be narrowed.
 	(save-restriction
@@ -400,8 +401,8 @@ The following commands are available:
 (defun gnus-server-copy-server (from to)
   (interactive
    (list
-    (unless (gnus-server-server-name)
-      (error "No server on the current line"))
+    (or (gnus-server-server-name)
+	(error "No server on the current line"))
     (read-string "Copy to: ")))
   (unless from
     (error "No server on current line"))
@@ -409,9 +410,10 @@ The following commands are available:
     (error "No name to copy to"))
   (when (assoc to gnus-server-alist)
     (error "%s already exists" to))
-  (unless (assoc from gnus-server-alist)
+  (unless (gnus-server-to-method from)
     (error "%s: no such server" from))
-  (let ((to-entry (gnus-copy-sequence (assoc from gnus-server-alist))))
+  (let ((to-entry (cons from (gnus-copy-sequence
+			      (gnus-server-to-method from)))))
     (setcar to-entry to)
     (setcar (nthcdr 2 to-entry) to)
     (push to-entry gnus-server-killed-servers)
@@ -512,7 +514,8 @@ The following commands are available:
    "\C-c\C-c" gnus-browse-exit
    "?" gnus-browse-describe-briefly
 
-   "\C-c\C-i" gnus-info-find-node))
+   "\C-c\C-i" gnus-info-find-node
+   "\C-c\C-b" gnus-bug))
 
 (defun gnus-browse-make-menu-bar ()
   (gnus-turn-off-edit-menu 'browse)
@@ -733,9 +736,10 @@ buffer.
     (if (not (gnus-check-backend-function 
 	      'request-regenerate (car (gnus-server-to-method server))))
 	(error "This backend doesn't support regeneration")
-      (gnus-message 5 "Requesing regeneration of %s..." server)
-      (when (gnus-request-regenerate server)
-	(gnus-message 5 "Requesing regeneration of %s...done" server)))))
+      (gnus-message 5 "Requesting regeneration of %s..." server)
+      (if (gnus-request-regenerate server)
+	  (gnus-message 5 "Requesting regeneration of %s...done" server)
+	(gnus-message 5 "Couldn't regenerate %s" server)))))
 					  
 (provide 'gnus-srvr)
 
