@@ -137,10 +137,13 @@ It is provided only to ease porting of broken FSF Emacs programs."
     (set-extent-face gnus-newsgroup-selected-overlay
 		     gnus-summary-selected-face)))
 
-(defvar gnus-xmas-force-redisplay t
+(defvar gnus-xmas-force-redisplay nil
   "If non-nil, force a redisplay before recentering the summary buffer.
 This is ugly, but it works around a bug in `window-displayed-height'.")
 
+(defun gnus-xmas-switch-horizontal-scrollbar-off ()
+  (set-specifier scrollbar-height (cons (current-buffer) 0)))
+
 (defun gnus-xmas-summary-recenter ()
   "\"Center\" point in the summary window.
 If `gnus-auto-center-summary' is nil, or the article buffer isn't
@@ -325,7 +328,9 @@ call it with the value of the `gnus-data' text property."
   (let ((event (next-command-event)))
     (sit-for 0)
     ;; We junk all non-key events.  Is this naughty?
-    (while (not (key-press-event-p event))
+    (while (not (or (key-press-event-p event)
+		    (button-press-event-p event)))
+      (dispatch-event event)
       (setq event (next-command-event)))
     (cons (and (key-press-event-p event) 
 	       (event-to-character event)) 
@@ -434,7 +439,6 @@ call it with the value of the `gnus-data' text property."
 	    (color-instance-rgb-components
 	     (make-color-instance color))))))
 
-
 (defun gnus-xmas-redefine ()
   "Redefine lots of Gnus functions for XEmacs."
   (fset 'gnus-summary-make-display-table 'ignore)
@@ -450,13 +454,13 @@ call it with the value of the `gnus-data' text property."
   (fset 'gnus-appt-select-lowest-window 
 	'gnus-xmas-appt-select-lowest-window)
   (fset 'gnus-mail-strip-quoted-names 'gnus-xmas-mail-strip-quoted-names)
-  (fset 'gnus-make-local-hook 'make-local-variable)
   (fset 'gnus-add-hook 'gnus-xmas-add-hook)
   (fset 'gnus-character-to-event 'character-to-event)
   (fset 'gnus-mode-line-buffer-identification
 	'gnus-xmas-mode-line-buffer-identification)
   (fset 'gnus-key-press-event-p 'key-press-event-p)
-
+  (fset 'gnus-region-active-p 'region-active-p)
+  
   (add-hook 'gnus-group-mode-hook 'gnus-xmas-group-menu-add)
   (add-hook 'gnus-summary-mode-hook 'gnus-xmas-summary-menu-add)
   (add-hook 'gnus-article-mode-hook 'gnus-xmas-article-menu-add)
@@ -473,6 +477,8 @@ call it with the value of the `gnus-data' text property."
   (add-hook 'gnus-group-mode-hook 'gnus-xmas-setup-group-toolbar)
   (add-hook 'gnus-summary-mode-hook 'gnus-xmas-setup-summary-toolbar)
 
+  (add-hook 'gnus-summary-mode-hook 'gnus-xmas-switch-horizontal-scrollbar-off)
+
   (when (and (<= emacs-major-version 19)
 	     (<= emacs-minor-version 13))
     (setq gnus-article-x-face-too-ugly (when (eq (device-type) 'tty)
@@ -738,22 +744,28 @@ XEmacs compatibility workaround."
 	   (glyph (make-glyph file)))
       (when (and (featurep 'x)
 		 (file-exists-p file))
-	(set-glyph-face glyph 'modeline-buffer-id))
-      (set-glyph-property glyph 'image (cons 'tty "Gnus:"))
-      glyph)))
+	(set-glyph-face glyph 'modeline-buffer-id)
+	(set-glyph-property glyph 'image (cons 'tty "Gnus:"))
+	glyph))))
 
 (defun gnus-xmas-mode-line-buffer-identification (line)
   (let ((line (car line))
 	chop)
-    (if (not (stringp line))
-	(list line)
-      (when (string-match "^Gnus:" line)
-	(setq chop (match-end 0))
-	(list 
-	 (if gnus-xmas-modeline-glyph
-	     (cons gnus-xmas-modeline-left-extent gnus-xmas-modeline-glyph)
-	   (cons gnus-xmas-modeline-left-extent (substring line 0 chop)))
-	 (cons gnus-xmas-modeline-right-extent (substring line chop)))))))
+    (cond
+     ;; This is some weird type of id.
+     ((not (stringp line))
+      (list line))
+     ;; This is non-standard, so we just pass it through.
+     ((not (string-match "^Gnus:" line))
+      (list line))
+     ;; We have a standard line, so we colorize and glyphize it a bit.
+     (t
+      (setq chop (match-end 0))
+      (list 
+       (if gnus-xmas-modeline-glyph
+	   (cons gnus-xmas-modeline-left-extent gnus-xmas-modeline-glyph)
+	 (cons gnus-xmas-modeline-left-extent (substring line 0 chop)))
+       (cons gnus-xmas-modeline-right-extent (substring line chop)))))))
 
 (defun gnus-xmas-splash ()
   (when (eq (device-type) 'x)
