@@ -3,7 +3,7 @@
 
 ;; Author: Masanobu UMEDA <umerin@flab.flab.fujitsu.junet>
 ;;	Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
-;; Keywords: news
+;; Keywords: news, mail
 
 ;; This file is part of GNU Emacs.
 
@@ -30,15 +30,21 @@
 
 (require 'custom)
 (require 'gnus-load)
+(require 'message)
 
 (defgroup gnus nil
   "The coffee-brewing, all singing, all dancing, kitchen sink newsreader."
-  :group 'emacs)
+  :group 'news
+  :group 'mail)
 
 (defgroup gnus-start nil
   "Starting your favorite newsreader."
   :group 'gnus)
 
+(defgroup gnus-start-server nil
+  "Server options at startup."
+  :group 'gnus-start)
+
 ;; These belong to gnus-group.el.
 (defgroup gnus-group nil
   "Group buffers."
@@ -50,6 +56,10 @@
   :link '(custom-manual "(gnus)Foreign Groups")
   :group 'gnus-group)
 
+(defgroup gnus-group-new nil
+  "Automatic subscription of new groups."
+  :group 'gnus-group)
+
 (defgroup gnus-group-levels nil
   "Group levels."
   :link '(custom-manual "(gnus)Group Levels")
@@ -152,6 +162,18 @@
   "Adaptive score files."
   :group 'gnus-score)
 
+(defgroup gnus-score-default nil
+  "Default values for score files."
+  :group 'gnus-score)
+
+(defgroup gnus-score-expire nil
+  "Expiring score rules."
+  :group 'gnus-score)
+
+(defgroup gnus-score-decay nil
+  "Decaying score rules."
+  :group 'gnus-score)
+
 (defgroup gnus-score-files nil
   "Score and kill file names."
   :group 'gnus-score
@@ -164,16 +186,22 @@
 ;; Other
 (defgroup gnus-visual nil
   "Options controling the visual fluff."
-  :group 'gnus)
-
-(defgroup gnus-mail-expire nil
-  "Expiring articles in mail backends."
-  :group 'gnus-mail)
+  :group 'gnus
+  :group 'faces)
 
 (defgroup gnus-files nil
   "Files used by Gnus."
   :group 'gnus)
 
+(defgroup gnus-dribble-file nil
+  "Auto save file."
+  :link '(custom-manual "(gnus)Auto Save")
+  :group 'gnus-files)
+
+(defgroup gnus-newsrc nil
+  "Storing Gnus state."
+  :group 'gnus-files)
+
 (defgroup gnus-server nil
   "Options related to newsservers and other servers used by Gnus."
   :group 'gnus)
@@ -198,7 +226,7 @@ is restarted, and sometimes reloaded."
   :link '(custom-manual "(gnus)Exiting Gnus")
   :group 'gnus)
 
-(defconst gnus-version-number "5.4.10"
+(defconst gnus-version-number "5.4.20"
   "Version number for this version of Gnus.")
 
 (defconst gnus-version (format "Gnus v%s" gnus-version-number)
@@ -230,7 +258,6 @@ be set in `.emacs' instead."
   (defalias 'gnus-topic-remove-excess-properties 'ignore)
   (defalias 'gnus-appt-select-lowest-window 'appt-select-lowest-window)
   (defalias 'gnus-mail-strip-quoted-names 'mail-strip-quoted-names)
-  (defalias 'gnus-make-local-hook 'make-local-hook)
   (defalias 'gnus-add-hook 'add-hook)
   (defalias 'gnus-character-to-event 'identity)
   (defalias 'gnus-add-text-properties 'add-text-properties)
@@ -1175,7 +1202,7 @@ to be desirable; see the manual for further details."
   "*Groups in which to automatically mark read articles as expirable.
 If non-nil, this should be a regexp that should match all groups in
 which to perform auto-expiry.  This only makes sense for mail groups."
-  :group 'gnus-mail-expire
+  :group 'nnmail-expire
   :type '(choice (const nil)
 		 regexp))
 
@@ -1185,7 +1212,7 @@ Use with extreme caution.  All groups that match this regexp will be
 expiring - which means that all read articles will be deleted after
 \(say) one week.	 (This only goes for mail groups and the like, of
 course.)"
-  :group 'gnus-mail-expire
+  :group 'nnmail-expire
   :type '(choice (const nil)
 		 regexp))
 
@@ -1578,7 +1605,7 @@ gnus-newsrc-hashtb should be kept so that both hold the same information.")
       gnus-uu-decode-uu-and-save-view gnus-uu-decode-unshar-view
       gnus-uu-decode-unshar-and-save-view gnus-uu-decode-save-view
       gnus-uu-decode-binhex-view)
-     ("gnus-uu" gnus-uu-delete-work-dir)
+     ("gnus-uu" gnus-uu-delete-work-dir gnus-quote-arg-for-sh-or-csh)
      ("gnus-msg" (gnus-summary-send-map keymap)
       gnus-article-mail gnus-copy-article-buffer gnus-extended-version)
      ("gnus-msg" :interactive t
@@ -2284,6 +2311,15 @@ just the host name."
 	      group nil)))
     name))
 
+(defun gnus-narrow-to-body ()
+  "Narrow to the body of an article."
+  (narrow-to-region
+   (progn
+     (goto-char (point-min))
+     (or (search-forward "\n\n" nil t)
+	 (point-max)))
+   (point-max)))
+
 
 ;;;
 ;;; Kill file handling.
