@@ -218,7 +218,8 @@
      (save-excursion
        (set-buffer nnbabyl-mbox-buffer)
        (save-buffer)))
-   nnbabyl-mbox-file group
+   (file-name-directory nnbabyl-mbox-file)
+   group
    (lambda ()
      (save-excursion
        (let ((in-buf (current-buffer)))
@@ -332,6 +333,7 @@
        (save-excursion
 	 (while (re-search-backward "^X-Gnus-Newsgroup: " beg t)
 	   (delete-region (point) (progn (forward-line 1) (point)))))
+       (nnmail-cache-insert (nnmail-fetch-field "message-id"))
        (setq result (car (nnbabyl-save-mail
 			  (if (stringp group)
 			      (list (cons group (nnbabyl-active-number group)))
@@ -342,6 +344,7 @@
        (goto-char (match-end 0))
        (insert-buffer-substring buf)
        (when last
+	 (nnmail-cache-insert (nnmail-fetch-field "message-id"))
 	 (save-buffer)
 	 (nnmail-save-active nnbabyl-group-alist nnbabyl-active-file))
        result))))
