@@ -92,7 +92,7 @@
   (nnml-retrieve-headers 0 nndir-current-group 0 0)
   (nnmh-request-article 0 nndir-current-group 0 0)
   (nnmh-request-group nndir-current-group 0 0)
-  (nnmh-close-group nndir-current-group 0)
+  (nnml-close-group nndir-current-group 0)
   (nnmh-request-list (nnoo-current-server 'nndir) nndir-directory)
   (nnmh-request-newsgroups (nnoo-current-server 'nndir) nndir-directory))
 
