@@ -89,6 +89,7 @@ time saver for large mailboxes.")
 (defvoo nnfolder-group-alist nil)
 (defvoo nnfolder-buffer-alist nil)
 (defvoo nnfolder-scantime-alist nil)
+(defvoo nnfolder-active-timestamp nil)
 
 
 
@@ -134,8 +135,8 @@ time saver for large mailboxes.")
 
 (deffoo nnfolder-open-server (server &optional defs)
   (nnoo-change-server 'nnfolder server defs)
-  (when (not (file-exists-p nnfolder-directory))
-    (gnus-make-directory nnfolder-directory))
+  (nnmail-activate 'nnfolder t)
+  (gnus-make-directory nnfolder-directory)
   (cond 
    ((not (file-exists-p nnfolder-directory))
     (nnfolder-close-server)
@@ -145,6 +146,7 @@ time saver for large mailboxes.")
     (nnfolder-close-server)
     (nnheader-report 'nnfolder "Not a directory: %s" nnfolder-directory))
    (t
+    (nnmail-activate 'nnfolder)
     (nnheader-report 'nnfolder "Opened server %s using directory %s"
 		     server nnfolder-directory)
     t)))
@@ -191,11 +193,11 @@ time saver for large mailboxes.")
 		    (point) (progn (end-of-line) (point)))))))))))
 
 (deffoo nnfolder-request-group (group &optional server dont-check)
+  (nnfolder-possibly-change-group group server)
   (save-excursion
     (nnmail-activate 'nnfolder)
     (if (not (assoc group nnfolder-group-alist))
 	(nnheader-report 'nnfolder "No such group: %s" group)
-      (nnfolder-possibly-change-group group server)
       (if dont-check
 	  (progn 
 	    (nnheader-report 'nnfolder "Selected group %s" group)
@@ -367,7 +369,7 @@ time saver for large mailboxes.")
   (nnfolder-possibly-change-group group server)
   (nnmail-check-syntax)
   (let ((buf (current-buffer))
-	result)
+	result art-group)
     (goto-char (point-min))
     (when (looking-at "X-From-Line: ")
       (replace-match "From "))
@@ -380,14 +382,18 @@ time saver for large mailboxes.")
        (forward-line -1)
        (while (re-search-backward (concat "^" nnfolder-article-marker) nil t)
 	 (delete-region (point) (progn (forward-line 1) (point))))
+       (nnmail-cache-insert (nnmail-fetch-field "message-id"))
        (setq result
 	     (car (nnfolder-save-mail
 		   (if (stringp group)
 		       (list (cons group (nnfolder-active-number group)))
-		     (nnmail-article-group 'nnfolder-active-number))))))
-     (save-excursion
-       (set-buffer nnfolder-current-buffer)
-       (and last (nnfolder-save-buffer))))
+		     (setq art-group
+			   (nnmail-article-group 'nnfolder-active-number)))))))
+     (when last
+       (save-excursion
+	 (nnfolder-possibly-change-folder (or (car art-group) group))
+	 (nnfolder-save-buffer)
+	 (nnmail-cache-close))))
     (nnmail-save-active nnfolder-group-alist nnfolder-active-file)
     (unless result
       (nnheader-report 'nnfolder "Couldn't store article"))
@@ -467,65 +473,51 @@ time saver for large mailboxes.")
 	     (point))
 	 (point-max))))))
 
-;; When scanning, we're not looking t immediately switch into the group - if
-;; we know our information is up to date, don't even bother reading the file.
 (defun nnfolder-possibly-change-group (group &optional server scanning)
+  ;; Change servers.
   (when (and server
 	     (not (nnfolder-server-opened server)))
     (nnfolder-open-server server))
-  (when (and group (or nnfolder-current-buffer
-		       (not (equal group nnfolder-current-group))))
-    (gnus-make-directory (directory-file-name nnfolder-directory))
-    (nnfolder-possibly-activate-groups nil)
-    (or (assoc group nnfolder-group-alist)
-	(not (file-exists-p
-	      (nnfolder-group-pathname group)))
-	(progn
-	  (push (list group (cons 1 0)) nnfolder-group-alist)
-	  (nnmail-save-active nnfolder-group-alist nnfolder-active-file)))
+  ;; Change group.
+  (when (and group
+	     (not (equal group nnfolder-current-group)))
+    (nnmail-activate 'nnfolder)
+    (when (and (not (assoc group nnfolder-group-alist))
+	       (not (file-exists-p
+		     (nnfolder-group-pathname group))))
+      ;; The group doesn't exist, so we create a new entry for it.
+      (push (list group (cons 1 0)) nnfolder-group-alist)
+      (nnmail-save-active nnfolder-group-alist nnfolder-active-file))
+    
     (let (inf file)
-      (if (and (equal group nnfolder-current-group)
-	       nnfolder-current-buffer
-	       (buffer-name nnfolder-current-buffer))
-	  ()
-	(setq nnfolder-current-group group)
+      ;; If we have to change groups, see if we don't already have the
+      ;; folder in memory.  If we do, verify the modtime and destroy
+      ;; the folder if needed so we can rescan it.
+      (when (setq inf (assoc group nnfolder-buffer-alist))
+	(setq nnfolder-current-buffer (nth 1 inf)))
 
-	;; If we have to change groups, see if we don't already have the mbox
-	;; in memory.  If we do, verify the modtime and destroy the mbox if
-	;; needed so we can rescan it.
-	(when (setq inf (assoc group nnfolder-buffer-alist))
-	  (setq nnfolder-current-buffer (nth 1 inf)))
+      ;; If the buffer is not live, make sure it isn't in the alist.  If it
+      ;; is live, verify that nobody else has touched the file since last
+      ;; time.
+      (when (and nnfolder-current-buffer
+		 (not (gnus-buffer-live-p nnfolder-current-buffer)))
+	(setq nnfolder-buffer-alist (delq inf nnfolder-buffer-alist)
+	      nnfolder-current-buffer nil))
 
-	;; If the buffer is not live, make sure it isn't in the alist.  If it
-	;; is live, verify that nobody else has touched the file since last
-	;; time.
-	(when (or (not (and nnfolder-current-buffer
-			    (buffer-name nnfolder-current-buffer)))
-		  (not (and (bufferp nnfolder-current-buffer)
-			    (verify-visited-file-modtime 
-			     nnfolder-current-buffer))))
-	  (when (and nnfolder-current-buffer
-		     (buffer-name nnfolder-current-buffer)
-		     (bufferp nnfolder-current-buffer))
-	    (kill-buffer nnfolder-current-buffer))
-	  (setq nnfolder-buffer-alist (delq inf nnfolder-buffer-alist))
-	  (setq inf nil))
-      
-	(unless inf
-	  (save-excursion
-	    (setq file (nnfolder-group-pathname group))
-	    (unless (file-directory-p (file-truename file))
-	      (unless (file-exists-p file)
-		(gnus-make-directory (file-name-directory file))
-		(nnmail-write-region 1 1 file t 'nomesg))
-	      (setq nnfolder-current-group group)
-	      (setq nnfolder-current-buffer
-		    (nnfolder-read-folder file scanning))
-	      (when nnfolder-current-buffer 
-		(set-buffer nnfolder-current-buffer)
-		(push (list group nnfolder-current-buffer)
-		      nnfolder-buffer-alist)))))))
-    (setq nnfolder-current-group group)))
+      (setq nnfolder-current-group group)
+    
+      (when (or (not nnfolder-current-buffer)
+		(not (verify-visited-file-modtime nnfolder-current-buffer)))
+	(save-excursion
+	  (setq file (nnfolder-group-pathname group))
+	  ;; See whether we need to create the new file.
+	  (unless (file-exists-p file)
+	    (gnus-make-directory (file-name-directory file))
+	    (nnmail-write-region 1 1 file t 'nomesg))
+	  (when (setq nnfolder-current-buffer (nnfolder-read-folder group))
+	    (set-buffer nnfolder-current-buffer)
+	    (push (list group nnfolder-current-buffer)
+		  nnfolder-buffer-alist)))))))
 
 (defun nnfolder-save-mail (group-art-list)
   "Called narrowed to an article."
@@ -550,35 +542,28 @@ time saver for large mailboxes.")
     (run-hooks 'nnfolder-prepare-save-mail-hook)
 
     ;; Insert the mail into each of the destination groups.
-    (while group-art-list
-      (setq group-art (car group-art-list)
-	    group-art-list (cdr group-art-list))
-
-      ;; Kill the previous newsgroup markers.
+    (while (setq group-art (pop group-art-list))
+      ;; Kill any previous newsgroup markers.
       (goto-char (point-min))
       (search-forward "\n\n" nil t)
       (forward-line -1)
       (while (search-backward (concat "\n" nnfolder-article-marker) nil t)
 	(delete-region (1+ (point)) (progn (forward-line 2) (point))))
 
-      (nnfolder-possibly-change-group (car group-art))
       ;; Insert the new newsgroup marker.
       (nnfolder-insert-newsgroup-line group-art)
-      (unless nnfolder-current-buffer
-	(nnfolder-close-group (car group-art))
-	(nnfolder-request-create-group (car group-art))
-	(nnfolder-possibly-change-group (car group-art)))
-      (let ((beg (point-min))
-	    (end (point-max))
-	    (obuf (current-buffer)))
-	(set-buffer nnfolder-current-buffer)
-	(goto-char (point-max))
-	(unless (eolp)
-	  (insert "\n"))
-	(unless (bobp)
-	  (insert "\n"))
-	(insert-buffer-substring obuf beg end)
-	(set-buffer obuf)))
+
+      (save-excursion
+	(let ((beg (point-min))
+	      (end (point-max))
+	      (obuf (current-buffer)))
+	  (nnfolder-possibly-change-folder (car group-art))
+	  (goto-char (point-max))
+	  (unless (eolp)
+	    (insert "\n"))
+	  (unless (bobp)
+	    (insert "\n"))
+	  (insert-buffer-substring obuf beg end))))
 
     ;; Did we save it anywhere?
     save-list))
@@ -591,32 +576,29 @@ time saver for large mailboxes.")
       (insert (format (concat nnfolder-article-marker "%d   %s\n")
 		      (cdr group-art) (current-time-string))))))
 
-(defun nnfolder-possibly-activate-groups (&optional group)
-  (save-excursion
-    ;; If we're looking for the activation of a specific group, find out
-    ;; its real name and switch to it.
-    (when group
-      (nnfolder-possibly-change-group group))
-    ;; If the group alist isn't active, activate it now.
-    (nnmail-activate 'nnfolder)))
-
 (defun nnfolder-active-number (group)
-  (when group
-    (save-excursion 
-      ;; Find the next article number in GROUP.
-      (prog1
-	  (let ((active (cadr (assoc group nnfolder-group-alist))))
-	    (if active
-		(setcdr active (1+ (cdr active)))
-	      ;; This group is new, so we create a new entry for it.
-	      ;; This might be a bit naughty... creating groups on the drop of
-	      ;; a hat, but I don't know...
-	      (push (list group (setq active (cons 1 1)))
-		    nnfolder-group-alist))
-	    (cdr active))
-	(nnmail-save-active nnfolder-group-alist nnfolder-active-file)
-	(nnfolder-possibly-activate-groups group)))))
+  ;; Find the next article number in GROUP.
+  (let ((active (cadr (assoc group nnfolder-group-alist))))
+    (if active
+	(setcdr active (1+ (cdr active)))
+      ;; This group is new, so we create a new entry for it.
+      ;; This might be a bit naughty... creating groups on the drop of
+      ;; a hat, but I don't know...
+      (push (list group (setq active (cons 1 1)))
+	    nnfolder-group-alist))
+    (cdr active)))
 
+(defun nnfolder-possibly-change-folder (group)
+  (let ((inf (assoc group nnfolder-buffer-alist)))
+    (if (and inf
+	     (gnus-buffer-live-p (cadr inf)))
+	(set-buffer (cadr inf))
+      (when inf
+	(setq nnfolder-buffer-alist (delq inf nnfolder-buffer-alist)))
+      (when nnfolder-group-alist
+	(nnmail-save-active nnfolder-group-alist nnfolder-active-file))
+      (push (list group (nnfolder-read-folder group))
+	    nnfolder-buffer-alist))))
 
 ;; This method has a problem if you've accidentally let the active list get
 ;; out of sync with the files.  This could happen, say, if you've
@@ -633,36 +615,27 @@ time saver for large mailboxes.")
 ;; shouldn't cost us much extra time at all, but will be a lot less
 ;; vulnerable to glitches between the mbox and the active file.
 
-(defun nnfolder-read-folder (file &optional scanning)
-  ;; This is an attempt at a serious shortcut - don't even read in the file
-  ;; if we know we've seen it since the last time it was touched.
-  (let ((scantime (cadr (assoc nnfolder-current-group 
-			       nnfolder-scantime-alist)))
-	(modtime (nth 5 (file-attributes file))))
-    (if (and scanning scantime
-	     (eq (car scantime) (car modtime))
-	     (eq (cdr scantime) (cadr modtime)))
-	nil
+(defun nnfolder-read-folder (group)
+  (let* ((file (nnfolder-group-pathname group))
+	 (buffer (set-buffer (nnheader-find-file-noselect file))))
+    (if (equal (cadr (assoc group nnfolder-scantime-alist))
+	       (nth 5 (file-attributes file)))
+	;; This looks up-to-date, so we don't do any scanning.
+	buffer
+      ;; Parse the damn thing.
       (save-excursion
-	(nnfolder-possibly-activate-groups nil)
+	(nnmail-activate 'nnfolder)
 	;; Read in the file.
-	(set-buffer (setq nnfolder-current-buffer 
-			  (nnheader-find-file-noselect file)))
-	(buffer-disable-undo (current-buffer))
-	(setq buffer-read-only nil)
-	;; If the file hasn't been touched since the last time we scanned it,
-	;; don't bother doing anything with it.
 	(let ((delim (concat "^" message-unix-mail-delimiter))
 	      (marker (concat "\n" nnfolder-article-marker))
 	      (number "[0-9]+")
-	      (active (or (cadr (assoc nnfolder-current-group 
-				       nnfolder-group-alist))
-			  (cons 1 0)))
-	      (scantime (assoc nnfolder-current-group nnfolder-scantime-alist))
+	      (active (cadr (assoc group nnfolder-group-alist)))
+	      (scantime (assoc group nnfolder-scantime-alist))
 	      (minid (lsh -1 -1))
-	      maxid start end newscantime)
-
-	  (setq maxid (or (cdr active) 0))
+	      maxid start end newscantime
+	      buffer-read-only)
+	  (buffer-disable-undo (current-buffer))
+	  (setq maxid (cdr active))
 	  (goto-char (point-min))
 
 	  ;; Anytime the active number is 1 or 0, it is suspect.  In that
@@ -697,10 +670,9 @@ time saver for large mailboxes.")
 	  ;; Keep track of the active number on our own, and insert it back
 	  ;; into the active list when we're done.  Also, prime the pump to
 	  ;; cut down on the number of searches we do.
+	  (unless (nnmail-search-unix-mail-delim)
+	    (goto-char (point-max)))
 	  (setq end (point-marker))
-	  (set-marker end (or (and (nnmail-search-unix-mail-delim)
-				   (point))
-			      (point-max)))
 	  (while (not (= end (point-max)))
 	    (setq start (marker-position end))
 	    (goto-char end)
@@ -708,9 +680,9 @@ time saver for large mailboxes.")
 	    ;; them.  
 	    (while (looking-at delim)
 	      (forward-line 1))
-	    (set-marker end (or (and (nnmail-search-unix-mail-delim)
-				     (point))
-				(point-max)))
+	    (set-marker end (if (nnmail-search-unix-mail-delim)
+				(point)
+			      (point-max)))
 	    (goto-char start)
 	    (when (not (search-forward marker end t))
 	      (narrow-to-region start end)
@@ -719,6 +691,7 @@ time saver for large mailboxes.")
 	       (cons nil (nnfolder-active-number nnfolder-current-group)))
 	      (widen)))
 
+	  (set-marker end nil)
 	  ;; Make absolutely sure that the active list reflects reality!
 	  (nnmail-save-active nnfolder-group-alist nnfolder-active-file)
 	  ;; Set the scantime for this group.
