@@ -31,10 +31,43 @@
 (eval-when-compile (require 'cl))
 (require 'custom)
 
-(defgroup gnus-mail nil
-  "Mailreading.."
+(defgroup nnmail nil
+  "Reading mail with Gnus."
   :group 'gnus)
 
+(defgroup nnmail-retrieve nil
+  "Retrieving new mail."
+  :group 'nnmail)
+
+(defgroup nnmail-prepare nil
+  "Preparing (or mangling) new mail after retrival."
+  :group 'nnmail)
+
+(defgroup nnmail-duplicate nil
+  "Handling of duplicate mail messages."
+  :group 'nnmail)
+
+(defgroup nnmail-split nil
+  "Organizing the incomming mail in folders."
+  :group 'nnmail)
+
+(defgroup nnmail-files nil
+  "Mail files."
+  :group 'gnus-files
+  :group 'nnmail)
+
+(defgroup nnmail-expire nil
+  "Expiring old mail."
+  :group 'nnmail)
+
+(defgroup nnmail-procmail nil
+  "Interfacing with procmail and other mail agents."
+  :group 'nnmail)
+
+(defgroup nnmail-various nil
+  "Various mail options."
+  :group 'nnmail)
+
 (defcustom nnmail-split-methods
   '(("mail.misc" ""))
   "Incoming mail will be split according to this variable.
@@ -62,7 +95,7 @@ mail belongs in that group.
 The last element should always have \"\" as the regexp.
 
 This variable can also have a function as its value."
-  :group 'gnus-mail
+  :group 'nnmail-split
   :type '(choice (repeat :tag "Alist" (group (string :tag "Name") regexp))
 		 (function-item nnmail-split-fancy)
 		 (function :tag "Other")))
@@ -71,7 +104,7 @@ This variable can also have a function as its value."
 (defcustom nnmail-crosspost t
   "If non-nil, do crossposting if several split methods match the mail.
 If nil, the first match found will be used."
-  :group 'gnus-mail
+  :group 'nnmail-split
   :type 'boolean)
 
 ;; Added by gord@enci.ucalgary.ca (Gordon Matzigkeit).
@@ -79,19 +112,20 @@ If nil, the first match found will be used."
   "If non-nil, nnmail will never delete the last expired article in a directory.  
 You may need to set this variable if other programs are putting
 new mail into folder numbers that Gnus has marked as expired."
-  :group 'gnus-mail
+  :group 'nnmail-procmail
+  :group 'nnmail-various
   :type 'boolean)
 
 (defcustom nnmail-use-long-file-names nil
   "If non-nil the mail backends will use long file and directory names.
 If nil, groups like \"mail.misc\" will end up in directories like
 \"mail/misc/\"."
-  :group 'gnus-mail
+  :group 'nnmail-files
   :type 'boolean)
 
 (defcustom nnmail-default-file-modes 384
   "Set the mode bits of all new mail files to this integer."
-  :group 'gnus-mail
+  :group 'nnmail-files
   :type 'integer)
 
 (defcustom nnmail-expiry-wait 7
@@ -99,7 +133,7 @@ If nil, groups like \"mail.misc\" will end up in directories like
 This variable can either be a number (which will be interpreted as a
 number of days) -- this doesn't have to be an integer.  This variable
 can also be `immediate' and `never'."
-  :group 'gnus-mail
+  :group 'nnmail-expire
   :type '(choice (const immediate)
 		 (integer :tag "days")
 		 (const never)))
@@ -119,7 +153,7 @@ Eg.:
               ((string-match \"junk\" newsgroup) 1)
 	      ((string-match \"important\" newsgroup) 'never)
 	      (t 7))))"
-  :group 'gnus-mail
+  :group 'nnmail-expire
   :type '(choice (const :tag "nnmail-expiry-wait" nil)
 		 (function :format "%v" nnmail-)))
 
@@ -133,41 +167,41 @@ If this variable is a list, all files mentioned in this list will be
 used as incoming mailboxes.
 If this variable is a directory (i. e., it's name ends with a \"/\"),
 treat all files in that directory as incoming spool files."
-  :group 'gnus-mail
+  :group 'nnmail-files
   :type 'file)
 
 (defcustom nnmail-crash-box "~/.gnus-crash-box"
   "File where Gnus will store mail while processing it."
-  :group 'gnus-mail
+  :group 'nnmail-files
   :type 'file)
 
 (defcustom nnmail-use-procmail nil
   "*If non-nil, the mail backends will look in `nnmail-procmail-directory' for spool files.
 The file(s) in `nnmail-spool-file' will also be read."
-  :group 'gnus-mail
+  :group 'nnmail-procmail
   :type 'boolean)
 
 (defcustom nnmail-procmail-directory "~/incoming/"
   "*When using procmail (and the like), incoming mail is put in this directory.
 The Gnus mail backends will read the mail from this directory."
-  :group 'gnus-mail
+  :group 'nnmail-procmail
   :type 'directory)
 
 (defcustom nnmail-procmail-suffix "\\.spool"
   "*Suffix of files created by procmail (and the like).
 This variable might be a suffix-regexp to match the suffixes of
 several files - eg. \".spool[0-9]*\"."
-  :group 'gnus-mail
+  :group 'nnmail-procmail
   :type 'regexp)
 
 (defcustom nnmail-resplit-incoming nil
   "*If non-nil, re-split incoming procmail sorted mail."
-  :group 'gnus-mail
+  :group 'nnmail-procmail
   :type 'boolean)
 
 (defcustom nnmail-delete-file-function 'delete-file
   "Function called to delete files in some mail backends."
-  :group 'gnus-mail
+  :group 'nnmail-files
   :type 'function)
 
 (defcustom nnmail-crosspost-link-function 'add-name-to-file
@@ -175,7 +209,7 @@ several files - eg. \".spool[0-9]*\"."
 This is `add-name-to-file' by default, which means that crossposts
 will use hard links.  If your file system doesn't allow hard
 links, you could set this variable to `copy-file' instead."
-  :group 'gnus-mail
+  :group 'nnmail-files
   :type '(radio (function-item add-name-to-file)
 		(function-item copy-file)
 		(function :tag "Other")))
@@ -187,12 +221,13 @@ The default is \"movemail\".
 This can also be a function.  In that case, the function will be
 called with two parameters -- the name of the INBOX file, and the file
 to be moved to."
-  :group 'gnus-mail
+  :group 'nnmail-files
+  :group 'nnmail-retrieve
   :type 'string)
 
 (defcustom nnmail-pop-password-required nil
   "*Non-nil if a password is required when reading mail using POP."
-  :group 'gnus-mail
+  :group 'nnmail-retrieve
   :type 'boolean)
 
 (defcustom nnmail-read-incoming-hook 
@@ -224,43 +259,44 @@ If you use `display-time', you could use something like this:
 	    ;; the flag that says you have mail.
 	    (when (eq (process-status \"display-time\") 'run)
 	      (display-time-filter display-time-process \"\"))))"
-  :group 'gnus-mail
+  :group 'nnmail-prepare
   :type 'hook)
 
 ;; Suggested by Erik Selberg <speed@cs.washington.edu>.
 (defcustom nnmail-prepare-incoming-hook nil
   "Hook called before treating incoming mail.
 The hook is run in a buffer with all the new, incoming mail."
-  :group 'gnus-mail
+  :group 'nnmail-prepare
   :type 'hook)
 
 (defcustom nnmail-prepare-incoming-header-hook nil
   "Hook called narrowed to the headers of each message.
 This can be used to remove excessive spaces (and stuff like
 that) from the headers before splitting and saving the messages."
-  :group 'gnus-mail
+  :group 'nnmail-prepare
   :type 'hook)
 
 (defcustom nnmail-prepare-incoming-message-hook nil
   "Hook called narrowed to each message."
-  :group 'gnus-mail
+  :group 'nnmail-prepare
   :type 'hook)
 
 (defcustom nnmail-list-identifiers nil
   "Regexp that matches list identifiers to be removed.
 This can also be a list of regexps."
-  :group 'gnus-mail
-  :type '(choice regexp
+  :group 'nnmail-prepare
+  :type '(choice (const :tag "none" nil)
+		 regexp
 		 (repeat regexp)))
 
 (defcustom nnmail-pre-get-new-mail-hook nil
   "Hook called just before starting to handle new incoming mail."
-  :group 'gnus-mail
+  :group 'nnmail-retrieve
   :type 'hook)
 
 (defcustom nnmail-post-get-new-mail-hook nil
   "Hook called just after finishing handling new incoming mail."
-  :group 'gnus-mail
+  :group 'nnmail-retrieve
   :type 'hook)
 
 (defcustom nnmail-split-hook nil
@@ -268,13 +304,15 @@ This can also be a list of regexps."
 The functions in this hook are free to modify the buffer
 contents in any way they choose -- the buffer contents are
 discarded after running the split process."
-  :group 'gnus-mail
+  :group 'nnmail-split
   :type 'hook)
 
 ;; Suggested by Mejia Pablo J <pjm9806@usl.edu>.
 (defcustom nnmail-tmp-directory nil
-  "*If non-nil, use this directory for temporary storage when reading incoming mail."
-  :group 'gnus-mail
+  "*If non-nil, use this directory for temporary storage.
+Used when reading incoming mail."
+  :group 'nnmail-files
+  :group 'nnmail-retrieve
   :type '(choice (const :tag "default" nil)
 		 (directory :format "%v")))
 
@@ -282,7 +320,7 @@ discarded after running the split process."
   "*The number of the articles which indicates a large newsgroup.
 If the number of the articles is greater than the value, verbose
 messages will be shown to indicate the current status."
-  :group 'gnus-mail
+  :group 'nnmail-various
   :type 'integer)
 
 (defcustom nnmail-split-fancy "mail.misc"
@@ -309,7 +347,7 @@ GROUP: Mail will be stored in GROUP (a string).
 
 FIELD must match a complete field name.  VALUE must match a complete
 word according to the `nnmail-split-fancy-syntax-table' syntax table.
-You can use .* in the regexps to match partial field names or words.
+You can use \".*\" in the regexps to match partial field names or words.
 
 FIELD and VALUE can also be lisp symbols, in that case they are expanded
 as specified in `nnmail-split-abbrev-alist'.
@@ -338,7 +376,7 @@ Example:
 	     (any \"larsi@ifi\\\\.uio\\\\.no\" \"people.Lars Magne Ingebrigtsen\"))
 	  ;; Unmatched mail goes to the catch all group.
 	  \"misc.misc\"))"
-  :group 'gnus-mail
+  :group 'nnmail-split
   ;; Sigh!
   :type 'sexp)
 
@@ -346,26 +384,27 @@ Example:
   '((any . "from\\|to\\|cc\\|sender\\|apparently-to\\|resent-from\\|resent-to\\|resent-cc")
     (mail . "mailer-daemon\\|postmaster\\|uucp"))
   "Alist of abbreviations allowed in `nnmail-split-fancy'."
-  :group 'gnus-mail
+  :group 'nnmail-split
   :type '(repeat (cons :format "%v" symbol regexp)))
 
 (defcustom nnmail-delete-incoming t
   "*If non-nil, the mail backends will delete incoming files after
 splitting."
-  :group 'gnus-mail
+  :group 'nnmail-retrieve
   :type 'boolean)
 
 (defcustom nnmail-message-id-cache-length 1000
   "*The approximate number of Message-IDs nnmail will keep in its cache.
 If this variable is nil, no checking on duplicate messages will be
 performed."
-  :group 'gnus-mail
+  :group 'nnmail-duplicate
   :type '(choice (const :tag "disable" nil)
 		 (integer :format "%v")))
 
 (defcustom nnmail-message-id-cache-file "~/.nnmail-cache"
   "*The file name of the nnmail Message-ID cache."
-  :group 'gnus-mail
+  :group 'nnmail-duplicate
+  :group 'nnmail-files
   :type 'file)
 
 (defcustom nnmail-treat-duplicates 'warn
@@ -378,7 +417,7 @@ and `delete', which means that nnmail will delete duplicated mails.
 This variable can also be a function.  It will be called from a buffer
 narrowed to the article in question with the Message-ID as a
 parameter.  It should return nil, `warn' or `delete'."
-  :group 'gnus-mail
+  :group 'nnmail-duplicate
   :type '(choice (const :tag "off" nil)
 		 (const warn)
 		 (const delete)))
@@ -493,7 +532,7 @@ parameter.  It should return nil, `warn' or `delete'."
       (delete-file nnmail-crash-box))
     (let ((inbox (file-truename (expand-file-name inbox)))
 	  (tofile (file-truename (expand-file-name nnmail-crash-box)))
-	  movemail popmail errors)
+	  movemail popmail errors result)
       (if (setq popmail (string-match
 			 "^po:" (file-name-nondirectory inbox)))
 	  (setq inbox (file-name-nondirectory inbox))
@@ -542,21 +581,32 @@ parameter.  It should return nil, `warn' or `delete'."
 		(buffer-disable-undo errors)
 		(let ((default-directory "/"))
 		  (if (nnheader-functionp nnmail-movemail-program)
-		      (funcall nnmail-movemail-program inbox tofile)
-		    (apply 
-		     'call-process
-		     (append
-		      (list
-		       (expand-file-name 
-			nnmail-movemail-program exec-directory)
-		       nil errors nil inbox tofile)
-		      (when nnmail-internal-password
-			(list nnmail-internal-password))))))
-		(if (not (buffer-modified-p errors))
+		      (condition-case err
+			  (progn
+			    (funcall nnmail-movemail-program inbox tofile)
+			    (setq result 0))
+			(error
+			 (save-excursion
+			   (set-buffer errors)
+			   (insert (prin1-to-string err))
+			   (setq result 255))))
+		    (setq result
+			  (apply 
+			   'call-process
+			   (append
+			    (list
+			     (expand-file-name 
+			      nnmail-movemail-program exec-directory)
+			     nil errors nil inbox tofile)
+			    (when nnmail-internal-password
+			      (list nnmail-internal-password)))))))
+		(if (and (not (buffer-modified-p errors))
+			 (zerop result))
 		    ;; No output => movemail won
 		    (progn
 		      (unless popmail
-			(set-file-modes tofile nnmail-default-file-modes))
+			(when (file-exists-p tofile)
+			  (set-file-modes tofile nnmail-default-file-modes)))
 		      (push inbox nnmail-moved-inboxes))
 		  (set-buffer errors)
 		  ;; There may be a warning about older revisions.  We
@@ -565,7 +615,8 @@ parameter.  It should return nil, `warn' or `delete'."
 		  (if (search-forward "older revision" nil t)
 		      (progn
 			(unless popmail
-			  (set-file-modes tofile nnmail-default-file-modes))
+			  (when (file-exists-p tofile)
+			    (set-file-modes tofile nnmail-default-file-modes)))
 			(push inbox nnmail-moved-inboxes))
 		    ;; Probably a real error.
 		    (subst-char-in-region (point-min) (point-max) ?\n ?\  )
@@ -576,8 +627,8 @@ parameter.  It should return nil, `warn' or `delete'."
 		    (when (looking-at "movemail: ")
 		      (delete-region (point-min) (match-end 0)))
 		    (unless (yes-or-no-p
-			     (format "movemail: %s.  Continue? "
-				     (buffer-string)))
+			     (format "movemail: %s (%d return).  Continue? "
+				     (buffer-string) result))
 		      (error "%s" (buffer-string)))
 		    (setq tofile nil)))))))
 	(message "Getting mail from %s...done" inbox)
@@ -666,12 +717,13 @@ is a spool.  If not using procmail, return GROUP."
       (goto-char (point-max))
       ;; Find the Message-ID header.
       (save-excursion
-	(if (re-search-backward "^Message-ID:[ \t]*\\(<[^>]*>\\)" nil t)
+	(if (re-search-backward
+	     "^Message-ID[ \t]*:[ \n\t]*\\(<[^>]*>\\)" nil t)
 	    (setq message-id (buffer-substring (match-beginning 1)
 					       (match-end 1)))
 	  ;; There is no Message-ID here, so we create one.
 	  (save-excursion
-	    (when (re-search-backward "^Message-ID:" nil t)
+	    (when (re-search-backward "^Message-ID[ \t]*:" nil t)
 	      (beginning-of-line)
 	      (insert "Original-")))
 	  (forward-line -1)
@@ -740,7 +792,7 @@ is a spool.  If not using procmail, return GROUP."
 		       (forward-line 1)
 		       (while (looking-at ">From ")
 			 (forward-line 1))
-		       (looking-at "[^ \t:]+[ \t]*:")))
+		       (looking-at "[^ \n\t:]+[ \n\t]*:")))
 	    (setq found 'yes)))))
     (beginning-of-line)
     (eq found 'yes)))
@@ -769,7 +821,7 @@ is a spool.  If not using procmail, return GROUP."
 		       (forward-line 1)
 		       (while (looking-at ">From ")
 			 (forward-line 1))
-		       (looking-at "[^ \t:]+[ \t]*:")))
+		       (looking-at "[^ \n\t:]+[ \n\t]*:")))
 	    (setq found 'yes)))))
     (beginning-of-line)
     (eq found 'yes)))
@@ -799,10 +851,10 @@ is a spool.  If not using procmail, return GROUP."
 	     (point))))
 	;; Find the Message-ID header.
 	(goto-char (point-min))
-	(if (re-search-forward "^Message-ID:[ \t]*\\(<[^>]+>\\)" nil t)
+	(if (re-search-forward "^Message-ID[ \t]*:[ \n\t]*\\(<[^>]+>\\)" nil t)
 	    (setq message-id (match-string 1))
 	  (save-excursion
-	    (when (re-search-forward "^Message-ID:" nil t)
+	    (when (re-search-forward "^Message-ID[ \t]*:" nil t)
 	      (beginning-of-line)
 	      (insert "Original-")))
 	  ;; There is no Message-ID here, so we create one.
@@ -883,11 +935,11 @@ is a spool.  If not using procmail, return GROUP."
 	     (point))))
 	;; Find the Message-ID header.
 	(goto-char (point-min))
-	(if (re-search-forward "^Message-ID:[ \t]*\\(<[^>]+>\\)" nil t)
+	(if (re-search-forward "^Message-ID[ \t]*:[ \n\t]*\\(<[^>]+>\\)" nil t)
 	    (setq message-id (match-string 1))
 	  ;; There is no Message-ID here, so we create one.
 	  (save-excursion
-	    (when (re-search-backward "^Message-ID:" nil t)
+	    (when (re-search-backward "^Message-ID[ \t]*:" nil t)
 	      (beginning-of-line)
 	      (insert "Original-")))
 	  (forward-line 1)
@@ -1254,19 +1306,19 @@ See the documentation for the variable `nnmail-split-fancy' for documentation."
 		'nconc
 		(mapcar 
 		 (lambda (file)
-		   (if (file-directory-p file)
+		   (if (and (not (string-match "^po:" file))
+			    (file-directory-p file))
 		       (nnheader-directory-regular-files file)
 		     (list file)))
 		 nnmail-spool-file))
 	       procmails))
-	     ((and (stringp nnmail-spool-file)
-		   (not (file-directory-p nnmail-spool-file)))
-	      (cons nnmail-spool-file procmails))
-	     ((and (stringp nnmail-spool-file)
-		   (file-directory-p nnmail-spool-file))
-	      (nconc
-	       (nnheader-directory-regular-files nnmail-spool-file)
-	       procmails))
+	     ((stringp nnmail-spool-file)
+	      (if (and (not (string-match "^po:" nnmail-spool-file))
+		       (file-directory-p nnmail-spool-file))
+		  (nconc 
+		   (nnheader-directory-regular-files nnmail-spool-file)
+		   procmails)
+		(cons nnmail-spool-file procmails)))
 	     ((eq nnmail-spool-file 'pop)
 	      (cons (format "po:%s" (user-login-name)) procmails))
 	     (t
@@ -1298,19 +1350,8 @@ See the documentation for the variable `nnmail-split-fancy' for documentation."
 	(save-excursion
 	  (or (eq timestamp 'none)
 	      (set (intern (format "%s-active-timestamp" backend)) 
-;;; dmoore@ucsd.edu 25.10.96
-;;; it's not always the case that current-time
-;;; does correspond to changes in the file's time.  So just compare
-;;; the file's new time against its own previous time.
-;;;		   (current-time)
-		   file-time
-		   ))
-	  (funcall (intern (format "%s-request-list" backend)))
-;;; dmoore@ucsd.edu 25.10.96
-;;; BACKEND-request-list already does this itself!
-;;;	  (set (intern (format "%s-group-alist" backend)) 
-;;;	       (nnmail-get-active))
-	  ))
+		   file-time))
+	  (funcall (intern (format "%s-request-list" backend)))))
     t))
 
 (defun nnmail-message-id ()
@@ -1358,11 +1399,12 @@ See the documentation for the variable `nnmail-split-fancy' for documentation."
 			   nnmail-message-id-cache-file nil 'silent)
       (set-buffer-modified-p nil)
       (setq nnmail-cache-buffer nil)
-      ;;(kill-buffer (current-buffer))
-      )))
+      (kill-buffer (current-buffer)))))
 
 (defun nnmail-cache-insert (id)
   (when nnmail-treat-duplicates
+    (unless (gnus-buffer-live-p nnmail-cache-buffer)
+      (nnmail-cache-open))
     (save-excursion
       (set-buffer nnmail-cache-buffer)
       (goto-char (point-max))
@@ -1375,6 +1417,12 @@ See the documentation for the variable `nnmail-split-fancy' for documentation."
       (goto-char (point-max))
       (search-backward id nil t))))
 
+(defun nnmail-fetch-field (header)
+  (save-excursion
+    (save-restriction
+      (message-narrow-to-head)
+      (message-fetch-field header))))
+
 (defun nnmail-check-duplication (message-id func artnum-func)
   (run-hooks 'nnmail-prepare-incoming-message-hook)
   ;; If this is a duplicate message, then we do not save it.
@@ -1400,9 +1448,10 @@ See the documentation for the variable `nnmail-split-fancy' for documentation."
      ((eq action 'warn)
       ;; We insert a warning.
       (let ((case-fold-search t)
-	    (newid (nnmail-message-id)))
+	    (newid (concat "<" (message-unique-id)
+			   "@duplicate-message-id>")))
 	(goto-char (point-min))
-	(when (re-search-forward "^message-id:" nil t)
+	(when (re-search-forward "^message-id[ \t]*:" nil t)
 	  (beginning-of-line)
 	  (insert "Original-"))
 	(beginning-of-line)
@@ -1471,8 +1520,9 @@ See the documentation for the variable `nnmail-split-fancy' for documentation."
 		    (if nnmail-tmp-directory
 			(concat 
 			 (file-name-as-directory nnmail-tmp-directory)
-			 (file-name-nondirectory (concat temp "Incoming")))
-		      (concat temp "Incoming")))))
+			 (file-name-nondirectory
+			  (concat (file-name-as-directory temp) "Incoming")))
+		      (concat (file-name-as-directory temp) "Incoming")))))
 	    (rename-file nnmail-crash-box incoming t)
 	    (push incoming incomings))))
       ;; If we did indeed read any incoming spools, we save all info. 
@@ -1530,7 +1580,8 @@ If ARGS, PROMPT is used as an argument to `format'."
     (unless nnmail-read-passwd
       (if (load "passwd" t)
 	  (setq nnmail-read-passwd 'read-passwd)
-	(autoload 'ange-ftp-read-passwd "ange-ftp")
+	(unless (fboundp 'ange-ftp-read-passwd)
+	  (autoload 'ange-ftp-read-passwd "ange-ftp"))
 	(setq nnmail-read-passwd 'ange-ftp-read-passwd)))
     (funcall nnmail-read-passwd prompt)))
 
@@ -1539,7 +1590,7 @@ If ARGS, PROMPT is used as an argument to `format'."
   (save-restriction
     (message-narrow-to-head)
     (let ((case-fold-search t))
-      (unless (re-search-forward "^Message-ID:" nil t)
+      (unless (re-search-forward "^Message-ID[ \t]*:" nil t)
 	(insert "Message-ID: " (nnmail-message-id) "\n")))))
 
 (defun nnmail-write-region (start end filename &optional append visit lockname)
@@ -1630,7 +1681,14 @@ If ARGS, PROMPT is used as an argument to `format'."
 	(setq found t
 	      his nil)))
     found))
-	
+
+(defun nnmail-pop3-movemail (inbox crashbox)
+  "Function to move mail from INBOX on a pop3 server to file CRASHBOX."
+  (require 'pop3)
+  (let ((pop3-maildrop
+         (substring inbox (match-end (string-match "^po:" inbox)))))
+    (pop3-movemail crashbox)))
+
 (run-hooks 'nnmail-load-hook)
 	    
 (provide 'nnmail)
