@@ -194,7 +194,8 @@
      (save-excursion
        (set-buffer nnmbox-mbox-buffer)
        (save-buffer)))
-   nnmbox-mbox-file group
+   (file-name-directory nnmbox-mbox-file)
+   group
    (lambda ()
      (save-excursion
        (let ((in-buf (current-buffer)))
@@ -303,6 +304,7 @@
        (forward-line -1)
        (while (re-search-backward "^X-Gnus-Newsgroup: " nil t)
 	 (delete-region (point) (progn (forward-line 1) (point))))
+       (nnmail-cache-insert (nnmail-fetch-field "message-id"))
        (setq result (nnmbox-save-mail
 		     (if (stringp group)
 			 (list (cons group (nnmbox-active-number group)))
@@ -311,9 +313,10 @@
        (set-buffer nnmbox-mbox-buffer)
        (goto-char (point-max))
        (insert-buffer-substring buf)
-       (and last (save-buffer))
-       result)
-     (nnmail-save-active nnmbox-group-alist nnmbox-active-file))
+       (when last
+	 (nnmail-cache-close)
+	 (nnmail-save-active nnmbox-group-alist nnmbox-active-file)
+	 (save-buffer))))
     (car result)))
 
 (deffoo nnmbox-request-replace-article (article group buffer)
