@@ -228,7 +228,8 @@
 (deffoo nnmh-request-newgroups (date &optional server)
   (nnmh-request-list server))
 
-(deffoo nnmh-request-expire-articles (articles newsgroup &optional server force)
+(deffoo nnmh-request-expire-articles (articles newsgroup
+					       &optional server force)
   (nnmh-possibly-change-directory newsgroup server)
   (let* ((active-articles 
 	  (mapcar
@@ -273,6 +274,7 @@
      (nnmh-request-article article group server)
      (save-excursion
        (set-buffer buf)
+       (erase-buffer)
        (insert-buffer-substring nntp-server-buffer)
        (setq result (eval accept-form))
        (kill-buffer (current-buffer))
@@ -288,6 +290,7 @@
 (deffoo nnmh-request-accept-article (group &optional server last noinsert)
   (nnmh-possibly-change-directory group server)
   (nnmail-check-syntax)
+  (nnmail-save-active nnml-group-alist nnml-active-file)
   (if (stringp group)
       (and 
        (nnmail-activate 'nnmh)
@@ -297,7 +300,9 @@
     (and
      (nnmail-activate 'nnmh)
      (car (nnmh-save-mail (nnmail-article-group 'nnmh-active-number)
-			  noinsert)))))
+			  noinsert))))
+  (when last
+    (nnmail-cache-close)))
 
 (deffoo nnmh-request-replace-article (article group buffer)
   (nnmh-possibly-change-directory group)
