@@ -101,18 +101,13 @@ all.  This may very well take some time.")
 	    beg article)
 	(if (stringp (car sequence))
 	    'headers
-	  (unless nnml-article-file-alist
-	    (setq nnml-article-file-alist
-		  (nnheader-article-to-file-alist nnml-current-directory)))
 	  (if (nnml-retrieve-headers-with-nov sequence fetch-old)
 	      'nov
 	    (while sequence
 	      (setq article (car sequence))
-	      (setq file 
-		    (concat nnml-current-directory 
-			    (or (cdr (assq article nnml-article-file-alist))
-				"")))
-	      (when (and (file-exists-p file)
+	      (setq file (nnml-article-to-file article))
+	      (when (and file
+			 (file-exists-p file)
 			 (not (file-directory-p file)))
 		(insert (format "221 %d Article retrieved.\n" article))
 		(setq beg (point))
@@ -159,7 +154,8 @@ all.  This may very well take some time.")
 
 (defun nnml-request-regenerate (server)
   (nnml-possibly-change-directory nil server)
-  (nnml-generate-nov-databases))
+  (nnml-generate-nov-databases)
+  t)
 
 (deffoo nnml-request-article (id &optional group server buffer)
   (nnml-possibly-change-directory group server)
@@ -260,15 +256,8 @@ all.  This may very well take some time.")
 	 article rest mod-time number)
     (nnmail-activate 'nnml)
 
-    (unless nnml-article-file-alist
-      (setq nnml-article-file-alist
-	    (nnheader-article-to-file-alist nnml-current-directory)))
-
     (while (and articles is-old)
-      (when (setq article
-		  (assq (setq number (pop articles)) 
-			nnml-article-file-alist))
-	(setq article (concat nnml-current-directory (cdr article)))
+      (when (setq article (nnml-article-to-file (setq number (pop articles))))
 	(when (setq mod-time (nth 5 (file-attributes article)))
 	  (if (and (nnml-deletable-article-p group number)
 		   (setq is-old 
@@ -298,9 +287,7 @@ all.  This may very well take some time.")
   (let ((buf (get-buffer-create " *nnml move*"))
 	result)
     (nnml-possibly-change-directory group server)
-    (unless nnml-article-file-alist
-      (setq nnml-article-file-alist
-	    (nnheader-article-to-file-alist nnml-current-directory)))
+    (nnml-update-file-alist)
     (and 
      (nnml-deletable-article-p group article)
      (nnml-request-article article group server)
@@ -314,17 +301,19 @@ all.  This may very well take some time.")
        (nnml-possibly-change-directory group server)
        (condition-case ()
 	   (funcall nnmail-delete-file-function
-		    (concat nnml-current-directory 
-			    (int-to-string article)))
+		    (nnml-article-to-file  article))
 	 (file-error nil))
        (nnml-nov-delete-article group article)
-       (and last (nnml-save-nov))))
+       (when last
+	 (nnml-save-nov)
+	 (nnmail-save-active nnml-group-alist nnml-active-file))))
     result))
 
 (deffoo nnml-request-accept-article (group &optional server last)
   (nnml-possibly-change-directory group server)
   (nnmail-check-syntax)
   (let (result)
+    (nnmail-cache-insert (nnmail-fetch-field "message-id"))
     (if (stringp group)
 	(and 
 	 (nnmail-activate 'nnml)
@@ -337,9 +326,10 @@ all.  This may very well take some time.")
        (nnmail-activate 'nnml)
        (setq result (car (nnml-save-mail
 			  (nnmail-article-group 'nnml-active-number))))
-       (progn
+       (when last
 	 (nnmail-save-active nnml-group-alist nnml-active-file)
-	 (and last (nnml-save-nov)))))
+	 (nnmail-cache-close)
+	 (nnml-save-nov))))
     result))
 
 (deffoo nnml-request-replace-article (article group buffer)
@@ -354,8 +344,9 @@ all.  This may very well take some time.")
 		(progn
 		  (nnmail-write-region 
 		   (point-min) (point-max)
-		   (concat nnml-current-directory
-			   (int-to-string article))
+		   (or (nnml-article-to-file article)
+		       (concat nnml-current-directory
+			       (int-to-string article)))
 		   nil (if (nnheader-be-verbose 5) nil 'nomesg))
 		  t)
 	      (error nil))
@@ -463,18 +454,15 @@ all.  This may very well take some time.")
 ;;; Internal functions.
 
 (defun nnml-article-to-file (article)
-  (unless nnml-article-file-alist
-    (setq nnml-article-file-alist
-	  (nnheader-article-to-file-alist nnml-current-directory)))
+  (nnml-update-file-alist)
   (let (file)
     (when (setq file (cdr (assq article nnml-article-file-alist)))
       (concat nnml-current-directory file))))
 
 (defun nnml-deletable-article-p (group article)
   "Say whether ARTICLE in GROUP can be deleted."
-  (let (file path)
-    (when (setq file (cdr (assq article nnml-article-file-alist)))
-      (setq path (concat nnml-current-directory file))
+  (let (path)
+    (when (setq path (nnml-article-to-file article))
       (when (file-writable-p path)
 	(or (not nnmail-keep-last-article)
 	    (not (eq (cdr (nth 1 (assoc group nnml-group-alist)))
@@ -774,11 +762,24 @@ all.  This may very well take some time.")
 (defun nnml-nov-delete-article (group article)
   (save-excursion
     (set-buffer (nnml-open-nov group))
-    (goto-char (point-min))
-    (when (re-search-forward (concat "^" (int-to-string article) "\t") nil t)
-      (delete-region (match-beginning 0) (progn (forward-line 1) (point))))
+    (when (nnheader-find-nov-line article)
+      (delete-region (point) (progn (forward-line 1) (point)))
+      (when (bobp)
+	(let ((active (cadr (assoc group nnml-group-alist)))
+	      num)
+	  (when active
+	    (if (eobp)
+		(setf (car active) (1+ (cdr active)))
+	      (when (and (setq num (ignore-errors (read (current-buffer))))
+			 (numberp num))
+		(setf (car active) num)))))))
     t))
 
+(defun nnml-update-file-alist ()
+  (unless nnml-article-file-alist
+    (setq nnml-article-file-alist
+	  (nnheader-article-to-file-alist nnml-current-directory))))
+
 (provide 'nnml)
 
 ;;; nnml.el ends here
