@@ -233,7 +233,8 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 	(re-search-backward "^[0-9]" nil t)
 	(when (looking-at "^[23]")
 	  (while (progn
-		   (goto-char (- (point-max) 3))
+		   (goto-char (point-max))
+		   (forward-line -1)
 		   (not (looking-at "^\\.\r?\n")))
 	    (nntp-accept-response)))
 	(and (numberp nntp-large-newsgroup)
@@ -548,8 +549,7 @@ It will prompt for a password."
 
 (defun nntp-send-authinfo-from-file ()
   "Send the AUTHINFO to the nntp server.
-This function is supposed to be called from `nntp-server-opened-hook'.
-It will prompt for a password."
+This function is supposed to be called from `nntp-server-opened-hook'."
   (when (file-exists-p "~/.nntp-authinfo")
     (nnheader-temp-write nil
       (insert-file-contents "~/.nntp-authinfo")
@@ -820,7 +820,8 @@ It will prompt for a password."
   (save-excursion
     (set-buffer (or (nntp-find-connection-buffer nntp-server-buffer)
 		    nntp-server-buffer))
-    (let ((len (/ (point-max) 1024)))
+    (let ((len (/ (point-max) 1024))
+	  message-log-max)
       (unless (< len 10)
 	(setq nntp-have-messaged t)
 	(nnheader-message 7 "nntp read: %dk" len)))
@@ -1087,8 +1088,7 @@ It will prompt for a password."
     (nntp-wait-for-string "^\r*200")
     (beginning-of-line)
     (delete-region (point-min) (point))
-    proc)
-  )
+    proc))
 
 (defun nntp-find-group-and-number ()
   (save-excursion
