@@ -1,10 +1,10 @@
 ;;; widget.el --- a library of user interface components.
 ;;
-;; Copyright (C) 1996 Free Software Foundation, Inc.
+;; Copyright (C) 1996, 1997 Free Software Foundation, Inc.
 ;;
 ;; Author: Per Abrahamsen <abraham@dina.kvl.dk>
 ;; Keywords: help, extensions, faces, hypermedia
-;; Version: 1.24
+;; Version: 1.55
 ;; X-URL: http://www.dina.kvl.dk/~abraham/custom/
 
 ;;; Commentary:
@@ -12,7 +12,7 @@
 ;; If you want to use this code, please visit the URL above.
 ;;
 ;; This file only contain the code needed to define new widget types.
-;; Everything else is autoloaded from `widget-edit.el'.
+;; Everything else is autoloaded from `wid-edit.el'.
 
 ;;; Code:
 
@@ -27,7 +27,7 @@
 	     (set (car keywords) (car keywords)))
 	 (setq keywords (cdr keywords)))))))
 
-(define-widget-keywords :valid-regexp
+(define-widget-keywords :tag-glyph :off-glyph :on-glyph :valid-regexp
   :secret :sample-face :sample-face-get :case-fold :widget-doc 
   :create :convert-widget :format :value-create :offset :extra-offset
   :tag :doc :from :to :args :value :value-from :value-to :action
@@ -42,10 +42,12 @@
   :hide-rear-space) 
 
 ;; These autoloads should be deleted when the file is added to Emacs.
-(autoload 'widget-create "widget-edit")
-(autoload 'widget-insert "widget-edit")
+(unless (fboundp 'load-gc)
+  (autoload 'widget-create "wid-edit")
+  (autoload 'widget-insert "wid-edit")
+  (autoload 'widget-browse "wid-browse" nil t)
+  (autoload 'widget-browse-at "wid-browse" nil t))
 
-;;;###autoload
 (defun define-widget (name class doc &rest args)
   "Define a new widget type named NAME from CLASS.
 
